@@ -157,6 +157,8 @@ class TestSystemAPIs(unittest.TestCase):
                     if not ps_part.mountpoint:
                         # this is usually a CD-ROM with no disk inserted
                         break
+                    if 'cdrom' in ps_part.opts:
+                        break
                     try:
                         usage = psutil.disk_usage(ps_part.mountpoint)
                     except OSError as err:
@@ -178,6 +180,8 @@ class TestSystemAPIs(unittest.TestCase):
 
     def test_disk_usage(self):
         for disk in psutil.disk_partitions():
+            if 'cdrom' in disk.opts:
+                continue
             sys_value = win32api.GetDiskFreeSpaceEx(disk.mountpoint)
             psutil_value = psutil.disk_usage(disk.mountpoint)
             self.assertAlmostEqual(sys_value[0], psutil_value.free,
@@ -518,15 +522,15 @@ class TestProcess(unittest.TestCase):
         import ctypes.wintypes
         PROCESS_QUERY_INFORMATION = 0x400
         handle = ctypes.windll.kernel32.OpenProcess(
-            PROCESS_QUERY_INFORMATION, 0, os.getpid())
+            PROCESS_QUERY_INFORMATION, 0, self.pid)
         self.addCleanup(ctypes.windll.kernel32.CloseHandle, handle)
+
         hndcnt = ctypes.wintypes.DWORD()
         ctypes.windll.kernel32.GetProcessHandleCount(
             handle, ctypes.byref(hndcnt))
         sys_value = hndcnt.value
-        psutil_value = psutil.Process().num_handles()
-        ctypes.windll.kernel32.CloseHandle(handle)
-        self.assertEqual(psutil_value, sys_value + 1)
+        psutil_value = psutil.Process(self.pid).num_handles()
+        self.assertEqual(psutil_value, sys_value)
 
 
 @unittest.skipIf(not WINDOWS, "WINDOWS only")
@@ -673,6 +677,22 @@ class TestDualProcessImplementation(unittest.TestCase):
                              num_handles)
             assert fun.called
 
+    def test_cmdline(self):
+        from psutil._pswindows import ACCESS_DENIED_ERRSET
+        for pid in psutil.pids():
+            try:
+                a = cext.proc_cmdline(pid, use_peb=True)
+                b = cext.proc_cmdline(pid, use_peb=False)
+            except OSError as err:
+                if err.errno in ACCESS_DENIED_ERRSET:
+                    pass
+                elif err.errno == errno.ESRCH:
+                    pass  # NSP
+                else:
+                    raise
+            else:
+                self.assertEqual(a, b)
+
 
 @unittest.skipIf(not WINDOWS, "WINDOWS only")
 class RemoteProcessTestCase(unittest.TestCase):
@@ -759,9 +779,10 @@ class RemoteProcessTestCase(unittest.TestCase):
 
     def test_environ_64(self):
         p = psutil.Process(self.proc64.pid)
-        e = p.environ()
-        self.assertIn("THINK_OF_A_NUMBER", e)
-        self.assertEquals(e["THINK_OF_A_NUMBER"], str(os.getpid()))
+        try:
+            p.environ()
+        except psutil.AccessDenied:
+            pass
 
 
 # ===================================================================
